#include <iostream>
#include "tableau.hpp"
#include "vecteur.hpp"
#include <vector>

const int N = 4; // Taille des matrices considerees


/////////////////////

// Produit de deux matrices, avec des tableaux.

//Cette fonction prend en argument trois matrices A, B et prod, et suppose qu'elles sont toutes les trois de dimension NxN. La produit de AxB est recopié dans la variable prod.  
void produit_tab(double **A, double **B, double **prod)
{
  for (int i=0; i<N; i++)
	for (int j=0; j<N; j++)
	  {
		prod[i][j] = 0;
		for (int k=0; k<N; k++)
		  prod[i][j] += A[i][k]*B[k][j];
	  }
}



/////////////////////

// Produit de deux matrices, avec des vecteurs.

typedef std::vector<std::vector<double> > matrix;
// Cette ligne définit le type "matrix" qui sera synonyme
// de "std::vector<std::vector<double> >", un peu long.

matrix produit_vect(matrix const & A, matrix const & B)
{
  matrix prod;
  prod.resize(N);
  for (int i=0; i<N; i++)
	prod[i].resize(N);
  
  for (int i=0; i<N; i++)
	for (int j=0; j<N; j++)
	  {
		prod[i][j] = 0;
		for (int k=0; k<N; k++)
		  prod[i][j] += A[i][k]*B[k][j];
	  }

  return prod;
}

/////////////////////////////////////////:

int main(void)
{
  std::mt19937 gen(678);
  

  // La version "tableaux"
  double **M = new double*[N];
  for (int i=0; i<N; i++)
	{
	  M[i] = new double[N];
	  remplit_tableau(M[i], N, gen); // On remplit la ieme ligne du tableau
	}

  std::cout << "Une matrice dont les coefficients sont tires selon des lois uniformes sur [0,1] :\n";
  for (int i=0; i<N; i++)
	{
	  for (int j=0; j<N; j++)
		std::cout << M[i][j] << "  ";
	  std::cout << "\n";
	}

  // Une nouvelle matrice dans laquelle on va stocker MxM.
  double **P = new double*[N];
  for (int i=0; i<N; i++)
	P[i] = new double[N];

  produit_tab(M,M,P);

  
  std::cout << "\nSon carré :\n";
  for (int i=0; i<N; i++)
	{
	  for (int j=0; j<N; j++)
		std::cout << P[i][j] << "  ";
	  std::cout << "\n";
	}
  
  std::cout << "\n------------------------\n\n";
  

  // La version "vecteurs"

  matrix A;
  A.resize(N);
  for (int i=0; i<N; i++)
	A[i].resize(N);


  gen.seed(678);


  for (int i=0; i<N; i++)
	remplit_vecteur(A[i], gen); // On remplit la ieme ligne
  
  
  std::cout << "Le même calcul, en utilisant des vecteurs plutôt que des tableaux :\nLa matrice :\n";
  for (int i=0; i<N; i++)
	{
	  for (int j=0; j<N; j++)
		std::cout << A[i][j] << "  ";
	  std::cout << "\n";
	}

  A = produit_vect(A,A);

  
  std::cout << "\nSon carré :\n";
  for (int i=0; i<N; i++)
	{
	  for (int j=0; j<N; j++)
		std::cout << A[i][j] << "  ";
	  std::cout << "\n";
	}



  return 0;

}
